/**
 * Copyright Copyright 2010 Simon Andrews
 *
 *    This file is part of Difference Tracker.
 *
 *    Difference Tracker is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Difference Tracker is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with Difference Tracker; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.DifferenceTracker.Config;

public class DifferenceTrackerPreferences {

	private static DifferenceTrackerPreferences prefs = new DifferenceTrackerPreferences(); 
	
	// A flag to say whether we want to produce an enormous output file
	private boolean produceFullOutput = false;
	
	// For the difference filter what is the min absolute diff we still count as changing
	private int minAbsoluteDiff = 10;
	
	// For the tracker how bright must a pixel be to track it
	private int minTrackedIntensity = 20;
	
	// How many pixels must be in a feature to try to track it
	private int minFeatureSize = 2;
	
	// How far apart can the same feature be in the its first two frames to track it
	private double initialFlexibility = 10;
	
	// How far off can a subsequent position prediction be to allow tracking to continue
	private double subsequentFlexibility = 5;
	
	// How many frames must a particle be tracked for to include it in the final stats
	private int minTrackLength = 4;
	
	// How many frames back should we look to decide if a pixel is moving or not
	private int differenceFrameOffset = 4;
	
	public static DifferenceTrackerPreferences getInstance () {
		return prefs;
	}
	
	private DifferenceTrackerPreferences () {}
	
	public boolean produceFullOutput () {
		return produceFullOutput;
	}
	public void setProduceFullOutput (boolean b) {
		produceFullOutput = b;
	}
	
	public int minTrackedIntensity () {
		return minTrackedIntensity;
	}
	public void setMinTrackedIntensity (int intensity) {
		if (intensity < 0) throw new IllegalArgumentException("Min intensity must be >=0");
		minTrackedIntensity = intensity;
	}
	
	public int minAbsoluteDiff () {
		return minAbsoluteDiff;
	}
	public void setMinAbsoluteDiff (int minDiff) {
		if (minDiff < 0) throw new IllegalArgumentException("Min difference must be > 0");
		minAbsoluteDiff = minDiff;
	}
	
	public int minFeatureSize () {
		return minFeatureSize;
	}
	public void setMinFeatureSize (int minSize) {
		if (minSize <= 0) throw new IllegalArgumentException("Min feature size must be > 0");
		minFeatureSize = minSize;		
	}
	
	public double initalFlexibility () {
		return initialFlexibility;
	}
	public void setInititalFlexibility (double flexibility) {
		if (flexibility <= 0) throw new IllegalArgumentException("Initial flexibility must be > 0");
		initialFlexibility = flexibility;		
	}
	
	public double subsequentFlexibility () {
		return subsequentFlexibility;
	}
	public void setSubsequentFlexibility (double flexibility) {
		if (flexibility <= 0) throw new IllegalArgumentException("Subsequent flexibility must be > 0");
		subsequentFlexibility = flexibility;		
	}
	
	public int minTrackLength () {
		return minTrackLength;
	}
	public void setMinTrackLength (int trackLength) {
		if (trackLength <= 0) throw new IllegalArgumentException("Track length must be > 0");
		minTrackLength = trackLength;				
	}
	
	public int differenceFrameOffset () {
		return differenceFrameOffset;
	}
	public void setDifferenceFrameOffset (int offset) {
		if (offset <=0) throw new IllegalArgumentException("Offset must be >0");
		differenceFrameOffset = offset;
	}
	
	
}
