/**
 * Copyright Copyright 2010 Simon Andrews
 *
 *    This file is part of Difference Tracker.
 *
 *    Difference Tracker is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Difference Tracker is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with Difference Tracker; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.DifferenceTracker.Config;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;


public class EditTrackerPrefs extends JDialog implements ActionListener {

	private DifferenceTrackerPreferences prefs = DifferenceTrackerPreferences.getInstance();

	private JCheckBox produceFullOutput;		
	private JTextField minTrackedIntensity;
	private JTextField minFeatureSize;
	private JTextField initialFlexibility;
	private JTextField subsequentFlexibility;
	private JTextField minTrackLength;
	
	private boolean wasCancelled = false;
	
	public EditTrackerPrefs () {
		
		setTitle("Tracker Preferences");
		setModal(true);
		
		JPanel prefsPanel = new JPanel();
		
		prefsPanel.setLayout(new GridLayout(6,2));
		
		// Full output
		JLabel fullOutputLabel = new JLabel("Produce full output");
		fullOutputLabel.setToolTipText("Produces extended output with all details of every tracked particle");
		prefsPanel.add(fullOutputLabel);
		
		produceFullOutput = new JCheckBox();
		produceFullOutput.setSelected(prefs.produceFullOutput());
		prefsPanel.add(produceFullOutput);
	
		// Min tracked intensity
		JLabel minIntensityLabel = new JLabel("Minimum tracked intensity");
		minIntensityLabel.setToolTipText("How bright does a pixel have to be to be included in the set of pixels to track");
		prefsPanel.add(minIntensityLabel);
		
		minTrackedIntensity = new JTextField(""+prefs.minTrackedIntensity());
		prefsPanel.add(minTrackedIntensity);
		
		// Min Feature Size
		JLabel minFeatureSizeLabel = new JLabel("Minimum feature size");
		minFeatureSizeLabel.setToolTipText("How many pixels need to cluster together to count as a particle to track");
		prefsPanel.add(minFeatureSizeLabel);
		
		minFeatureSize = new JTextField(""+prefs.minFeatureSize());
		prefsPanel.add(minFeatureSize);

		
		// Initial Flexibility
		JLabel initialFlexibilityLabel = new JLabel("Initial flexibility");
		initialFlexibilityLabel.setToolTipText("How far apart can two particles initially be to start considering them as a track");
		prefsPanel.add(initialFlexibilityLabel);
		
		initialFlexibility = new JTextField(""+prefs.initalFlexibility());
		prefsPanel.add(initialFlexibility);

		// Subsequent Flexibility
		JLabel subsequentFlexibilityLabel = new JLabel("Subsequent flexibility");
		subsequentFlexibilityLabel.setToolTipText("How away from its predicted position can a particle be to still form part of the same track");
		prefsPanel.add(subsequentFlexibilityLabel);
		
		subsequentFlexibility = new JTextField(""+prefs.subsequentFlexibility());
		prefsPanel.add(subsequentFlexibility);

		// Min track length
		JLabel minTrackLengthLabel = new JLabel("Min track length");
		minTrackLengthLabel.setToolTipText("How many frames must a particle be tracked for to count towards the overall statistics");
		prefsPanel.add(minTrackLengthLabel);
		
		minTrackLength = new JTextField(""+prefs.minTrackLength());
		prefsPanel.add(minTrackLength);

		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(new JScrollPane(prefsPanel),BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		JButton saveButton = new JButton("Run");
		saveButton.setActionCommand("run");
		saveButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		buttonPanel.add(saveButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setSize(400,200);
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		setLocationRelativeTo(null);
		setVisible(true);
	}

	public boolean wasCancelled () {
		return wasCancelled;
	}
	
	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("cancel")){
			wasCancelled = true;
			setVisible(false);
		}
		else if (ae.getActionCommand().equals("run")) {
			try {
				prefs.setProduceFullOutput(produceFullOutput.isSelected());
				prefs.setMinTrackedIntensity(Integer.parseInt(minTrackedIntensity.getText()));
				prefs.setMinFeatureSize(Integer.parseInt(minFeatureSize.getText()));
				prefs.setInititalFlexibility(Double.parseDouble(initialFlexibility.getText()));
				prefs.setSubsequentFlexibility(Double.parseDouble(subsequentFlexibility.getText()));
				prefs.setMinTrackLength(Integer.parseInt(minTrackLength.getText()));
			}
			catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(this, "All entries must be integers", "Error setting preferences", JOptionPane.ERROR_MESSAGE);
				return;
			}
			catch (IllegalArgumentException e) {
				JOptionPane.showMessageDialog(this, e.getMessage(), "Error setting preferences", JOptionPane.ERROR_MESSAGE);
				return;				
			}
			setVisible(false);
			dispose();
		}
		else {
			throw new IllegalArgumentException("Don't understand the action '"+ae.getActionCommand()+"'");
		}
		
	}
	
	
	
	
}
